@@ -9,45 +9,51 @@
 
 #include "ScVehicle.hpp"
 
+#include "../../../ride/TrackData.h"
+#include "../../../world/tile_element/TrackElement.h"
 #include "../ride/ScRide.hpp"
 
 #ifdef ENABLE_SCRIPTING
 
+using namespace OpenRCT2::Drawing;
+using namespace OpenRCT2::TrackMetaData;
+
 namespace OpenRCT2::Scripting
 {
-    static const DukEnumMap<Vehicle::Status> VehicleStatusMap({
-        { "moving_to_end_of_station", Vehicle::Status::MovingToEndOfStation },
-        { "waiting_for_passengers", Vehicle::Status::WaitingForPassengers },
-        { "waiting_to_depart", Vehicle::Status::WaitingToDepart },
-        { "departing", Vehicle::Status::Departing },
-        { "travelling", Vehicle::Status::Travelling },
-        { "arriving", Vehicle::Status::Arriving },
-        { "unloading_passengers", Vehicle::Status::UnloadingPassengers },
-        { "travelling_boat", Vehicle::Status::TravellingBoat },
-        { "crashing", Vehicle::Status::Crashing },
-        { "crashed", Vehicle::Status::Crashed },
-        { "travelling_dodgems", Vehicle::Status::TravellingDodgems },
-        { "swinging", Vehicle::Status::Swinging },
-        { "rotating", Vehicle::Status::Rotating },
-        { "ferris_wheel_rotating", Vehicle::Status::FerrisWheelRotating },
-        { "simulator_operating", Vehicle::Status::SimulatorOperating },
-        { "showing_film", Vehicle::Status::ShowingFilm },
-        { "space_rings_operating", Vehicle::Status::SpaceRingsOperating },
-        { "top_spin_operating", Vehicle::Status::TopSpinOperating },
-        { "haunted_house_operating", Vehicle::Status::HauntedHouseOperating },
-        { "doing_circus_show", Vehicle::Status::DoingCircusShow },
-        { "crooked_house_operating", Vehicle::Status::CrookedHouseOperating },
-        { "waiting_for_cable_lift", Vehicle::Status::WaitingForCableLift },
-        { "travelling_cable_lift", Vehicle::Status::TravellingCableLift },
-        { "stopping", Vehicle::Status::Stopping },
-        { "waiting_for_passengers_17", Vehicle::Status::WaitingForPassengers17 },
-        { "waiting_to_start", Vehicle::Status::WaitingToStart },
-        { "starting", Vehicle::Status::Starting },
-        { "operating_1a", Vehicle::Status::Operating1A },
-        { "stopping_1b", Vehicle::Status::Stopping1B },
-        { "unloading_passengers_1c", Vehicle::Status::UnloadingPassengers1C },
-        { "stopped_by_block_brake", Vehicle::Status::StoppedByBlockBrakes },
-    });
+    static const DukEnumMap<Vehicle::Status> VehicleStatusMap(
+        {
+            { "moving_to_end_of_station", Vehicle::Status::MovingToEndOfStation },
+            { "waiting_for_passengers", Vehicle::Status::WaitingForPassengers },
+            { "waiting_to_depart", Vehicle::Status::WaitingToDepart },
+            { "departing", Vehicle::Status::Departing },
+            { "travelling", Vehicle::Status::Travelling },
+            { "arriving", Vehicle::Status::Arriving },
+            { "unloading_passengers", Vehicle::Status::UnloadingPassengers },
+            { "travelling_boat", Vehicle::Status::TravellingBoat },
+            { "crashing", Vehicle::Status::Crashing },
+            { "crashed", Vehicle::Status::Crashed },
+            { "travelling_dodgems", Vehicle::Status::TravellingDodgems },
+            { "swinging", Vehicle::Status::Swinging },
+            { "rotating", Vehicle::Status::Rotating },
+            { "ferris_wheel_rotating", Vehicle::Status::FerrisWheelRotating },
+            { "simulator_operating", Vehicle::Status::SimulatorOperating },
+            { "showing_film", Vehicle::Status::ShowingFilm },
+            { "space_rings_operating", Vehicle::Status::SpaceRingsOperating },
+            { "top_spin_operating", Vehicle::Status::TopSpinOperating },
+            { "haunted_house_operating", Vehicle::Status::HauntedHouseOperating },
+            { "doing_circus_show", Vehicle::Status::DoingCircusShow },
+            { "crooked_house_operating", Vehicle::Status::CrookedHouseOperating },
+            { "waiting_for_cable_lift", Vehicle::Status::WaitingForCableLift },
+            { "travelling_cable_lift", Vehicle::Status::TravellingCableLift },
+            { "stopping", Vehicle::Status::Stopping },
+            { "waiting_for_passengers_17", Vehicle::Status::WaitingForPassengers17 },
+            { "waiting_to_start", Vehicle::Status::WaitingToStart },
+            { "starting", Vehicle::Status::Starting },
+            { "operating_1a", Vehicle::Status::Operating1A },
+            { "stopping_1b", Vehicle::Status::Stopping1B },
+            { "unloading_passengers_1c", Vehicle::Status::UnloadingPassengers1C },
+            { "stopped_by_block_brake", Vehicle::Status::StoppedByBlockBrakes },
+        });
 
     ScVehicle::ScVehicle(EntityId id)
         : ScEntity(id)
@@ -74,8 +80,10 @@ namespace OpenRCT2::Scripting
         dukglue_register_property(
             ctx, &ScVehicle::flag_get<VehicleFlags::CarIsReversed>, &ScVehicle::flag_set<VehicleFlags::CarIsReversed>,
             "isReversed");
+        dukglue_register_property(
+            ctx, &ScVehicle::flag_get<VehicleFlags::Crashed>, &ScVehicle::flag_set<VehicleFlags::Crashed>, "isCrashed");
         dukglue_register_property(ctx, &ScVehicle::colours_get, &ScVehicle::colours_set, "colours");
-        dukglue_register_property(ctx, &ScVehicle::trackLocation_get, &ScVehicle::trackLocation_set, "trackLocation");
+        dukglue_register_property(ctx, &ScVehicle::trackLocation_get, nullptr, "trackLocation");
         dukglue_register_property(ctx, &ScVehicle::trackProgress_get, nullptr, "trackProgress");
         dukglue_register_property(ctx, &ScVehicle::remainingDistance_get, nullptr, "remainingDistance");
         dukglue_register_property(ctx, &ScVehicle::subposition_get, nullptr, "subposition");
@@ -88,6 +96,7 @@ namespace OpenRCT2::Scripting
         dukglue_register_property(ctx, &ScVehicle::guests_get, nullptr, "guests");
         dukglue_register_property(ctx, &ScVehicle::gForces_get, nullptr, "gForces");
         dukglue_register_method(ctx, &ScVehicle::travelBy, "travelBy");
+        dukglue_register_method(ctx, &ScVehicle::moveToTrack, "moveToTrack");
     }
 
     Vehicle* ScVehicle::GetVehicle() const
@@ -107,6 +116,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->ride_subtype = value;
+            vehicle->Invalidate();
         }
     }
 
@@ -122,6 +132,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->vehicle_type = value;
+            vehicle->Invalidate();
         }
     }
 
@@ -137,6 +148,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->Pitch = value;
+            vehicle->Invalidate();
         }
     }
 
@@ -333,6 +345,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->bank_rotation = value;
+            vehicle->Invalidate();
         }
     }
 
@@ -358,6 +371,7 @@ namespace OpenRCT2::Scripting
             {
                 vehicle->ClearFlag(flag);
             }
+            vehicle->Invalidate();
         }
     }
 
@@ -378,6 +392,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->colours = FromDuk<VehicleColour>(value);
+            vehicle->Invalidate();
         }
     }
 
@@ -397,20 +412,6 @@ namespace OpenRCT2::Scripting
         }
         return ToDuk(ctx, nullptr);
     }
-    void ScVehicle::trackLocation_set(const DukValue& value)
-    {
-        ThrowIfGameStateNotMutable();
-        auto vehicle = GetVehicle();
-        if (vehicle != nullptr)
-        {
-            auto x = AsOrDefault(value["x"], 0);
-            auto y = AsOrDefault(value["y"], 0);
-            auto z = AsOrDefault(value["z"], 0);
-            vehicle->TrackLocation = CoordsXYZ(x, y, z);
-            vehicle->SetTrackDirection(AsOrDefault(value["direction"], 0));
-            vehicle->SetTrackType(static_cast<TrackElemType>(AsOrDefault(value["trackType"], 0)));
-        }
-    }
 
     uint16_t ScVehicle::trackProgress_get() const
     {
@@ -495,6 +496,7 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->spin_sprite = value;
+            vehicle->Invalidate();
         }
     }
 
@@ -543,8 +545,44 @@ namespace OpenRCT2::Scripting
         if (vehicle != nullptr)
         {
             vehicle->MoveRelativeDistance(value);
+            EntityTweener::Get().RemoveEntity(vehicle);
         }
     }
+
+    void ScVehicle::moveToTrack(int32_t x, int32_t y, int32_t elementIndex)
+    {
+        auto vehicle = GetVehicle();
+        if (vehicle == nullptr)
+            return;
+
+        CoordsXY coords = TileCoordsXY(x, y).ToCoordsXY();
+        auto el = MapGetNthElementAt(coords, elementIndex);
+        if (el == nullptr)
+            return;
+
+        auto origin = GetTrackSegmentOrigin(CoordsXYE(coords, el));
+        if (!origin)
+            return;
+
+        const auto& trackType = el->AsTrack()->GetTrackType();
+        const auto& ted = GetTrackElementDescriptor(trackType);
+        const auto& seq0 = ted.sequences[0].clearance;
+        const auto trackLoc = CoordsXYZ(origin->x + seq0.x, origin->y + seq0.y, origin->z + seq0.z);
+
+        vehicle->TrackLocation.x = trackLoc.x;
+        vehicle->TrackLocation.y = trackLoc.y;
+        vehicle->TrackLocation.z = trackLoc.z;
+        vehicle->SetTrackDirection(origin->direction);
+        vehicle->SetTrackType(trackType);
+
+        // Clip track progress to avoid being out of bounds of current piece
+        uint16_t trackTotalProgress = vehicle->GetTrackProgress();
+        if (trackTotalProgress && vehicle->track_progress >= trackTotalProgress)
+            vehicle->track_progress = trackTotalProgress - 1;
+
+        vehicle->UpdateTrackChange();
+        EntityTweener::Get().RemoveEntity(vehicle);
+    }
 } // namespace OpenRCT2::Scripting
 
 #endif
